from reahl.tofu import Fixture, uses
from reahl.tofu.pytestsupport import with_fixtures

from reahl.browsertools.browsertools import Browser, XPath

from reahl.doc.examples.tutorial.i18nexamplebootstrap.i18nexamplebootstrap import AddressBookUI, Address

from reahl.web_dev.fixtures import WebFixture


@uses(web_fixture=WebFixture)
class TranslationExampleFixture(Fixture):

    def new_browser(self):
        return Browser(self.web_fixture.new_wsgi_app(site_root=AddressBookUI))

    def create_addresses(self):
        addresses = [Address(name='friend %s' % i, email_address='friend%s@some.org' % i) for i in list(range(1, 5))]
        for address in addresses:
            address.save()
        return addresses


@with_fixtures(WebFixture, TranslationExampleFixture)
def demo_setup(sql_alchemy_fixture, translation_example_fixture):
    sql_alchemy_fixture.commit = True

    translation_example_fixture.create_addresses()


@with_fixtures(WebFixture, TranslationExampleFixture)
def test_translations(web_fixture, translation_example_fixture):
    """The user can choose between languages. The text for which translations exist change accordingly."""

    
    browser = translation_example_fixture.browser
    browser.open('/')
    assert browser.is_element_present(XPath.heading(1).with_text("Addresses"))
    assert browser.is_element_present(XPath.label().with_text("Name"))

    #go to the the translated page
    browser.click(XPath.link().with_text('Afrikaans'))
    assert browser.is_element_present(XPath.heading(1).with_text("Adresse"))
    assert browser.is_element_present(XPath.label().with_text("Naam"))




# In production this has to be set, to the name of the egg of your application:
reahlsystem.root_egg = 'hellodockernginx'   

# If using PostgreSQL:
reahlsystem.connection_uri = 'postgresql://hellodockernginx:hellodockernginx@database/hellodockernginx'

# If using MySQL:
#reahlsystem.connection_uri = 'mysql://hellodockernginx:hellodockernginx@database/hellodockernginx'

reahlsystem.debug = False


import os

# In production this has to be set, to the name of the egg of your application:
reahlsystem.root_egg = 'helloanywhere'   
reahlsystem.debug = False

# If using SQLite:
reahlsystem.connection_uri = 'sqlite:///%s/helloanywhere.db' % os.path.expanduser('~')

# If using MySql:
# reahlsystem.connection_uri = 'mysql://myname:mydatabasepassword@myname.mysql.pythonanywhere-services.com/helloanywhere'

# If using PostgreSQL:
#reahlsystem.connection_uri = 'postgresql://myname:mydatabasepassword@myname-somenumber.postgres.pythonanywhere-services.com/helloanywhere'





#Read here to create a paypal sandbox account:
# https://developer.paypal.com/docs/api/overview/#create-sandbox-accounts
import os
paymentpaypal.client_id = os.environ['PAYPAL_CLIENT_ID']
paymentpaypal.client_secret = os.environ['PAYPAL_CLIENT_SECRET']
paymentpaypal.sandboxed = True



from reahl.web.libraries import LibraryIndex, JQuery, JsCookie, JQueryUI, Underscore, HTML5Shiv, IE9, Reahl, Holder, ReahlBootstrap4Additions

from reahl.doc.examples.howtos.bootstrapsassmultihomed.bootstrapsassmultihomed import ThemedUI, CompiledBootstrap


web.site_root = ThemedUI
web.frontend_libraries = LibraryIndex(JQuery(), JsCookie(), JQueryUI(), Underscore(), HTML5Shiv(), IE9(), Reahl(), Holder(),
                         CompiledBootstrap(), ReahlBootstrap4Additions())





from reahl.web.fw import UserInterface
from reahl.web.ui import HTML5Page, HTMLWidget, Div, LiteralHTML
from reahl.web.plotly import Chart

import plotly.graph_objects as go

class GraphPage(HTML5Page):
    def __init__(self, view):
        super().__init__(view)

        fig1 = self.create_bar_chart_figure()
        self.body.add_child(Chart(view, fig1, 'bar'))

        fig2 = self.create_line_chart_figure()
        self.body.add_child(Chart(view, fig2, 'line'))

    def create_line_chart_figure(self):
        x = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
        fig = go.Figure()
        fig.add_trace(go.Scatter(x=x, y=[1000, 1500, 1360, 1450, 1470, 1500, 1700], name='first line'))
        fig.add_trace(go.Scatter(x=x, y=[100, 200, 300, 450, 530, 570, 600, 640, 630, 690, ], name='second line'))
        fig.update_layout(
            title="Line chart",
            hovermode="x unified",
            xaxis_title="X Axis Title",
            yaxis_title="Y Axis Title"
        )
        return fig

    def create_bar_chart_figure(self):
        fig = go.Figure()
        fig.add_trace(go.Bar(y=[2, 3, 1], x=['foo', 'bar', 'baz']))
        fig.update_layout(
            title="Bar chart",
            xaxis_title="X Axis Title",
            yaxis_title="Y Axis Title"
        )
        return fig


class PlotlyUI(UserInterface):
    def assemble(self):
        self.define_view('/', title='Home', page=GraphPage.factory())





from __future__ import print_function, unicode_literals, absolute_import, division

from reahl.component.modelinterface import ExposedNames, Event, Action, IntegerField, Choice, ChoiceField
from reahl.sqlalchemysupport import session_scoped, Base, Integer, Column
from reahl.web.fw import UserInterface
from reahl.web.bootstrap.page import HTML5Page
from reahl.web.bootstrap.forms import Form, SelectInput, InlineFormLayout, MarginLayout, TextInput
from reahl.web.bootstrap.ui import P, FieldSet
from reahl.web.bootstrap.grid import Container


@session_scoped
class Calculator(Base):
    __tablename__ = 'dynamic_content_select'

    id              = Column(Integer, primary_key=True)
    operandA        = Column(Integer)
    operandB        = Column(Integer)
    sum             = Column(Integer)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.operandA = 1
        self.operandB = 4
        self.recalculate()

    fields = ExposedNames()
    fields.operandA = lambda i: IntegerField(label='A')
    fields.operandB = lambda i: ChoiceField([Choice(4, IntegerField(label='4')),
                                       Choice(5, IntegerField(label='5')),
                                       Choice(6, IntegerField(label='6'))],
                                      label='B')

    events = ExposedNames()
    events.input_changed = lambda i: Event(action=Action(i.recalculate))

    def recalculate(self):
        self.sum = self.operandA + self.operandB


class DynamicContentForm(Form):
    def __init__(self, view):
        super().__init__(view, 'my_cool_form')

        calculator = Calculator.for_current_session()
        self.enable_refresh(on_refresh=calculator.events.input_changed)

        grouped_inputs = self.add_child(FieldSet(view, legend_text='Calculator'))
        grouped_inputs.use_layout(InlineFormLayout())

        grouped_inputs.layout.add_input(TextInput(self, calculator.fields.operandA, refresh_widget=self)).use_layout(MarginLayout(2, right=True))
        grouped_inputs.layout.add_input(SelectInput(self, calculator.fields.operandB, refresh_widget=self))

        self.add_child(P(self.view, text='Sum: %s' % calculator.sum))


class DynamicContentPage(HTML5Page):
    def __init__(self, view):
        super(DynamicContentPage, self).__init__(view)
        self.body.use_layout(Container())
        self.body.add_child(DynamicContentForm(view))


class DynamicContentUI(UserInterface):
    def assemble(self):
        self.define_view('/', title='Dynamic content demo', page=DynamicContentPage.factory())







from reahl.web.fw import UserInterface
from reahl.web.bootstrap.page import HTML5Page
from reahl.web.bootstrap.forms import Form, TextInput, FormLayout, Button
from reahl.component.modelinterface import secured, Action, Event
from reahl.component.modelinterface import ExposedNames, Field, ExposedNames


class AccessUI(UserInterface):
    def assemble(self):
        self.define_view('/', title='Access control demo', 
                              page=HomePage.factory())


class HomePage(HTML5Page):
    def __init__(self, view):
        super().__init__(view)
        self.body.add_child(CommentForm(view))

class Comment:
    def allowed_to_see(self):
        # this is hard-coded, but you can check business rules here
        return True 

    def allowed_to_write(self):
        # this is hard-coded, but you can check business rules here
        return False

    fields = ExposedNames()
    fields.greyed_out_field = lambda i: Field(label='Some data',
                                              default='a value you\'re allowed to see, but not edit, so it is greyed out',
                                              readable=Action(i.allowed_to_see),
                                              writable=Action(i.allowed_to_write))

    events = ExposedNames()
    events.greyed_out_event = lambda i: Event(label='Greyed out button', 
                                              action=Action(i.do_something))

    @secured(read_check=allowed_to_see, write_check=allowed_to_write)
    def do_something(self):
        pass


class CommentForm(Form):
    def __init__(self, view):
        super().__init__(view, 'myform')
        comment = Comment()

        self.use_layout(FormLayout())

        self.layout.add_input(TextInput(self, comment.fields.greyed_out_field))

        self.define_event_handler(comment.events.greyed_out_event)
        self.add_child(Button(self, comment.events.greyed_out_event))











from reahl.web.fw import UserInterface
from reahl.web.layout import PageLayout
from reahl.web.bootstrap.page import HTML5Page
from reahl.web.bootstrap.ui import P
from reahl.web.bootstrap.navs import Nav, TabLayout
from reahl.web.bootstrap.grid import ColumnLayout, ColumnOptions, ResponsiveSize, Container


class MyCustomPage(HTML5Page):
    def __init__(self, view, bookmarks):
        super().__init__(view)

        self.use_layout(PageLayout(document_layout=Container()))
        contents_layout = ColumnLayout(ColumnOptions('secondary', size=ResponsiveSize(md=3)),
                                       ColumnOptions('main', size=ResponsiveSize(md=9))).with_slots()
        self.layout.contents.use_layout(contents_layout)

        menu = Nav(view).use_layout(TabLayout()).with_bookmarks(bookmarks)
        self.layout.header.add_child(menu)


class SlotsUI(UserInterface):
    def assemble(self):

        home = self.define_view('/', title='Page 1')
        home.set_slot('main', P.factory(text='In this slot will be some main content for the view on /'))
        home.set_slot('secondary', P.factory(text='Some secondary content related to /'))

        another = self.define_view('/page2', title='Page 2')
        another.set_slot('main', P.factory(text='This could, for example, be where a photo gallery shows a large photo.'))
        another.set_slot('secondary', P.factory(text='Thumbnails will then sit on the side of the big photo.'))

        bookmarks = [home.as_bookmark(self), another.as_bookmark(self)]
        self.define_page(MyCustomPage, bookmarks)




# Copyright 2018-2023 Reahl Software Services (Pty) Ltd. All rights reserved.
#
#    This file is part of Reahl.
#
#    Reahl is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as
#    published by the Free Software Foundation; version 3 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


from reahl.web.fw import UserInterface, Widget, Url
from reahl.web.layout import PageLayout
from reahl.web.bootstrap.page import HTML5Page
from reahl.web.bootstrap.ui import A, Alert
from reahl.web.bootstrap.forms import ButtonLayout
from reahl.web.bootstrap.grid import Container, ColumnLayout, ColumnOptions, ResponsiveSize
from reahl.web.libraries import Library


class ThemedUI(UserInterface):
    def assemble(self):
        self.define_page(HTML5Page).use_layout(
            PageLayout(document_layout=Container(),
                      contents_layout=ColumnLayout(ColumnOptions('main', size=ResponsiveSize(lg=6))).with_slots()))
        home = self.define_view('/', title='Themed example')
        home.set_slot('main', SomeWidget.factory())


class SomeWidget(Widget):
    def __init__(self, view):
        super().__init__(view)

        self.add_child(Alert(view, 'This is an alert in danger color', severity='danger'))
        self.add_child(Alert(view, 'This is an alert in primary color', severity='primary'))
        self.add_child(A(view, Url('#'), description='Link styled as button')).use_layout(ButtonLayout(style='primary'))


class CompiledBootstrap(Library):
    def __init__(self):
        super().__init__('custom')
        self.egg_name = 'bootstrapsass'
        self.shipped_in_package = 'dist'
        self.files = [
                      'theme.css',
                      'main.js'
                      ]






from sqlalchemy import Column, Integer, UnicodeText

from reahl.sqlalchemysupport import Session, Base

from reahl.web.fw import UserInterface, Widget
from reahl.web.layout import PageLayout
from reahl.web.bootstrap.page import HTML5Page
from reahl.web.bootstrap.ui import Div, P, H
from reahl.web.bootstrap.forms import Form, TextInput, Button, FormLayout, FieldSet
from reahl.web.bootstrap.grid import ColumnLayout, ColumnOptions, ResponsiveSize, Container
from reahl.component.modelinterface import ExposedNames, EmailField, Field, Event, Action
from reahl.component.config import Configuration, ConfigSetting
from reahl.component.context import ExecutionContext


class AddressConfig(Configuration):
    filename = 'componentconfig.config.py'
    config_key = 'componentconfig'

    showheader = ConfigSetting(default=False, description='Whether the title should be shown as a heading too')


class AddressBookUI(UserInterface):
    def assemble(self):
        find = self.define_view('/', title='Addresses', page=AddressBookPage.factory())
        find.set_slot('main', AddressBookPanel.factory())


class AddressBookPage(HTML5Page):
    def __init__(self, view):
        super().__init__(view)
        self.use_layout(PageLayout(document_layout=Container()))
        contents_layout = ColumnLayout(ColumnOptions('main', ResponsiveSize(md=4))).with_slots()
        self.layout.contents.use_layout(contents_layout)


class AddressBookPanel(Div):
    def __init__(self, view):
        super().__init__(view)

        config = ExecutionContext.get_context().config
        if config.componentconfig.showheader:
            self.add_child(H(view, 1, text='Addresses'))
        
        for address in Session.query(Address).all():
            self.add_child(AddressBox(view, address))

        self.add_child(AddAddressForm(view))


class AddAddressForm(Form):
    def __init__(self, view):
        super().__init__(view, 'add_form')

        new_address = Address()

        grouped_inputs = self.add_child(FieldSet(view, legend_text='Add an address'))
        grouped_inputs.use_layout(FormLayout())
        grouped_inputs.layout.add_input(TextInput(self, new_address.fields.name))
        grouped_inputs.layout.add_input(TextInput(self, new_address.fields.email_address))

        self.define_event_handler(new_address.events.save)
        grouped_inputs.add_child(Button(self, new_address.events.save, style='primary'))


class AddressBox(Widget):
    def __init__(self, view, address):
        super().__init__(view)
        self.add_child(P(view, text='%s: %s' % (address.name, address.email_address)))


class Address(Base):
    __tablename__ = 'tutorial_componentconfigbootstrap_address'
    
    id            = Column(Integer, primary_key=True)
    email_address = Column(UnicodeText)
    name          = Column(UnicodeText)

    fields = ExposedNames()
    fields.name = lambda i: Field(label='Name', required=True)
    fields.email_address = lambda i: EmailField(label='Email', required=True)

    def save(self):
        Session.add(self)
        
    events = ExposedNames()
    events.save = lambda i: Event(label='Save', action=Action(i.save))






# Copyright 2021, 2022, 2023 Reahl Software Services (Pty) Ltd. All rights reserved.
#
#    This file is part of Reahl.
#
#    Reahl is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as
#    published by the Free Software Foundation; version 3 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.



from reahl.component.modelinterface import ExposedNames, Field, Event, Action, Choice, IntegerField
from reahl.component.exceptions import DomainException
from reahl.web.fw import UserInterface
from reahl.web.layout import PageLayout
from reahl.web.bootstrap.page import HTML5Page
from reahl.web.bootstrap.ui import FieldSet, Span
from reahl.web.bootstrap.grid import Container, ColumnLayout, ColumnOptions, ResponsiveSize
from reahl.web.bootstrap.forms import Form, FormLayout, TextInput, ChoiceField, InlineFormLayout, SelectInput

from sqlalchemy import Column, Integer, UnicodeText, orm
from reahl.sqlalchemysupport import Base, session_scoped


class DynamicUI(UserInterface):
    def assemble(self):
        self.define_page(HTML5Page).use_layout(PageLayout(document_layout=Container(),
                                                          contents_layout=ColumnLayout(
                                                              ColumnOptions('main', size=ResponsiveSize(lg=7))).with_slots()))
        home = self.define_view('/', title='Dynamic error demo')
        home.set_slot('main', CalculatorForm.factory())


class CalculatorForm(Form):
    def __init__(self, view):
        super().__init__(view, 'dynamic_content_error_form')
        self.use_layout(FormLayout())
        self.calculator = Calculator.for_current_session()

        try:
            self.enable_refresh(on_refresh=self.calculator.events.inputs_changed)
        except DomainException as ex:
            self.layout.add_alert_for_domain_exception(ex)

        controls = self.add_child(FieldSet(view).use_layout(InlineFormLayout()))
        self.add_inputs(controls)
        self.display_result(controls)

    def add_inputs(self, controls):
        operand_a_input = TextInput(self, self.calculator.fields.operand_a, refresh_widget=self)
        controls.layout.add_input(operand_a_input, hide_label=True)

        operator_input = SelectInput(self, self.calculator.fields.operator, refresh_widget=self)
        controls.layout.add_input(operator_input, hide_label=True)

        operand_b_input = TextInput(self, self.calculator.fields.operand_b, refresh_widget=self)
        controls.layout.add_input(operand_b_input, hide_label=True)

    def display_result(self, controls):
        if self.calculator.result is not None:
            message = '= %s' % self.calculator.result
        else:
            message = '= ---'
        controls.add_child(Span(self.view, text=message))


@session_scoped
class Calculator(Base):
    __tablename__ = 'dynamic_content_error_calculator'

    id              = Column(Integer, primary_key=True)
    operand_a       = Column(Integer)
    operand_b       = Column(Integer)
    operator        = Column(UnicodeText)
    result          = Column(Integer)

    fields = ExposedNames()
    fields.operand_a  = lambda i: IntegerField(label='A', required=True)
    fields.operand_b  = lambda i: IntegerField(label='B', required=True)
    fields.operator   = lambda i: ChoiceField([Choice('plus', Field(label='+')),
                                          Choice('divide', Field(label='÷'))], required=True)

    events = ExposedNames()
    events.inputs_changed = lambda i: Event(action=Action(i.recalculate))

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.operand_a = 1
        self.operand_b = 1
        self.operator = 'plus'
        self.recalculate()

    @property
    def is_divide_by_zero(self):
        return self.operator == 'divide' and self.operand_b == 0

    def recalculate(self):
        if self.is_divide_by_zero:
            self.result = None
            raise DomainException(message='I can\'t divide by 0')

        if self.operator == 'plus':
            self.result = self.operand_a + self.operand_b
        elif self.operator == 'divide':
            self.result = int(self.operand_a / self.operand_b)





from reahl.tofu import expected, NoException


from sqlalchemy import Column, Integer, UnicodeText

from reahl.sqlalchemysupport import Session, Base, metadata
from reahl.component.modelinterface import ExposedNames, EmailField, Field, Event, Action
from reahl.component.context import ExecutionContext


class Address(Base):
    __tablename__ = 'modeltests3_address'

    id            = Column(Integer, primary_key=True)
    email_address = Column(UnicodeText)
    name          = Column(UnicodeText)

    def save(self):
        Session.add(self)

    fields = ExposedNames()
    fields.name = lambda i: Field(label='Name', required=True)
    fields.email_address = lambda i: EmailField(label='Email', required=True)

    events = ExposedNames()
    events.save = lambda i: Event(label='Save', action=Action(i.save))


def test_reahl_additions():
    with ExecutionContext():

        try:
            metadata.bind = 'sqlite:///:memory:'
            metadata.create_all()

            address = Address()
            Session.add(address)
            email_field = address.fields.email_address

            # While a programmer would not usually write code like this,
            # it is useful to show how the framework can use Fields and Events
            # to obtain more information about a certain Field/Event:
            assert email_field.label == 'Email'

            # Fields are used (amongst other things) to validate user input:
            with expected(Exception):
                email_field.from_input('invalid email address')

            with expected(NoException):
                assert address.email_address is None
                email_field.from_input('valid@email.com')
                assert address.email_address == 'valid@email.com'

            # After input was given, the field is set on the object it belongs to:
            # (The value set is a marshalled version of the user input. In this case it is just
            #  a string again, but it could have been, for example an EmailAddress object,
            #  and Integer, or a Date.)
            assert address.email_address == 'valid@email.com'

        finally:
            metadata.bind = None




from __future__ import print_function, unicode_literals, absolute_import, division
from reahl.doc.examples.features.dynamiccontent.dynamiccontent import DynamicContentUI

web.site_root = DynamicContentUI





# To run this test do:
# pytest -s --pyargs reahl.doc.examples.howtos.pagerbootstrap.pagerbootstrap_dev.test_pagerbootstrap
#

from reahl.tofu import Fixture, uses
from reahl.tofu.pytestsupport import with_fixtures

from reahl.browsertools.browsertools import XPath
from reahl.doc.examples.howtos.pagerbootstrap.pagerbootstrap import AddressBookUI

from reahl.web_dev.fixtures import WebFixture


@uses(web_fixture=WebFixture)
class PagingFixture(Fixture):

    def new_browser(self):
        return self.web_fixture.driver_browser
        
    def new_wsgi_app(self):
        return self.web_fixture.new_wsgi_app(site_root=AddressBookUI, enable_js=True)

    def is_email_listed(self, email):
        return self.browser.is_element_present(XPath.paragraph().including_text(email))


@with_fixtures(WebFixture, PagingFixture)
def test_paging(web_fixture, paging_fixture):
    """Clicking on a different page in the pager changes the addresses listed without triggering a page load."""

    web_fixture.reahl_server.set_app(paging_fixture.wsgi_app)
    browser = paging_fixture.browser

    browser.open('/')

    assert paging_fixture.is_email_listed('friend0@some.org')
    assert not paging_fixture.is_email_listed('friend9@some.org')

    with browser.no_page_load_expected():
        browser.click(XPath.link().with_text('2'))

    assert not paging_fixture.is_email_listed('friend0@some.org')
    assert paging_fixture.is_email_listed('friend9@some.org')




# To run this test do:
# pytest --pyargs reahl.doc.examples.tutorial.accessbootstrap.accessbootstrap_dev.test_accessbootstrap
#
# To set up a demo database for playing with, do:
# pytest -o python_functions=demo_setup --pyargs reahl.doc.examples.tutorial.accessbootstrap.accessbootstrap_dev.test_accessbootstrap

from reahl.tofu import scenario, Fixture, uses
from reahl.tofu.pytestsupport import with_fixtures

from reahl.sqlalchemysupport import Session

from reahl.browsertools.browsertools import Browser, XPath

from reahl.doc.examples.tutorial.accessbootstrap.accessbootstrap import Address, AddressBook, AddressBookUI
from reahl.domain.systemaccountmodel import EmailAndPasswordSystemAccount

from reahl.sqlalchemysupport_dev.fixtures import SqlAlchemyFixture
from reahl.dev.fixtures import ReahlSystemFixture
from reahl.web_dev.fixtures import WebFixture


@uses(web_fixture=WebFixture)
class AccessUIFixture(Fixture):

    def new_browser(self):
        return Browser(self.web_fixture.new_wsgi_app(site_root=AddressBookUI, enable_js=True))

    def is_on_address_book_page_of(self, email):
        return self.browser.title == 'Address book of %s' % email


class AccessDomainFixture(Fixture):
    password = 'topsecret'

    def new_account(self, email='johndoe@some.org'):
        account = EmailAndPasswordSystemAccount(email=email)
        Session.add(account)
        account.set_new_password(account.email, self.password)
        account.activate()
        return account

    def new_address_book(self, owner=None):
        owner = owner or self.account
        address_book = AddressBook(owner=owner)
        Session.add(address_book)
        return address_book

    def new_other_account(self):
        return self.new_account(email='other@some.org')

    def new_other_address_book(self):
        return self.new_address_book(owner=self.other_account)


@with_fixtures(SqlAlchemyFixture, AccessDomainFixture)
def demo_setup(sql_alchemy_fixture, access_domain_fixture):
    sql_alchemy_fixture.commit = True

    access_domain_fixture.address_book
    john = access_domain_fixture.account
    jane = access_domain_fixture.new_account(email='janedoe@some.org')
    jane_book = access_domain_fixture.new_address_book(owner=jane)
    someone = access_domain_fixture.new_account(email='someone@some.org')
    someone_book = access_domain_fixture.new_address_book(owner=someone)
    someone_else = access_domain_fixture.new_account(email='someoneelse@some.org')
    someone_else_book = access_domain_fixture.new_address_book(owner=someone_else)

    jane_book.allow(john, can_add_addresses=True, can_edit_addresses=True)
    someone_book.allow(john, can_add_addresses=False, can_edit_addresses=True)
    someone_else_book.allow(john, can_add_addresses=False, can_edit_addresses=False)

    Address(address_book=jane_book, email_address='friend1@some.org', name='Friend1').save()
    Address(address_book=jane_book, email_address='friend2@some.org', name='Friend2').save()
    Address(address_book=jane_book, email_address='friend3@some.org', name='Friend3').save()
    Address(address_book=jane_book, email_address='friend4@some.org', name='Friend4').save()

    Address(address_book=someone_book, email_address='friend11@some.org', name='Friend11').save()
    Address(address_book=someone_book, email_address='friend12@some.org', name='Friend12').save()
    Address(address_book=someone_book, email_address='friend13@some.org', name='Friend13').save()
    Address(address_book=someone_book, email_address='friend14@some.org', name='Friend14').save()

    Address(address_book=someone_else_book, email_address='friend21@some.org', name='Friend21').save()
    Address(address_book=someone_else_book, email_address='friend22@some.org', name='Friend22').save()
    Address(address_book=someone_else_book, email_address='friend23@some.org', name='Friend23').save()
    Address(address_book=someone_else_book, email_address='friend24@some.org', name='Friend24').save()


@with_fixtures(SqlAlchemyFixture, AccessDomainFixture)
def test_separate_address_books(sql_alchemy_fixture, access_domain_fixture):
    """An Address is created in a particular AddressBook, which is owned by a SystemAccount."""


    account = access_domain_fixture.account
    address_book = access_domain_fixture.address_book
    other_address_book = access_domain_fixture.other_address_book

    # AddressBooks are owned
    assert address_book.owner is account
    assert other_address_book.owner is access_domain_fixture.other_account

    # Addresses live in specific AddressBooks
    assert address_book.addresses == []
    assert other_address_book.addresses == []

    address1 = Address(address_book=address_book, email_address='friend1@some.org', name='Friend1')
    address2 = Address(address_book=address_book, email_address='friend2@some.org', name='Friend2')

    address3 = Address(address_book=other_address_book, email_address='friend3@some.org', name='Friend3')

    for address in [address1, address2, address3]:
        address.save()

    assert address_book.addresses == [address1, address2]
    assert other_address_book.addresses == [address3]


@with_fixtures(SqlAlchemyFixture, AccessDomainFixture)
def test_collaborators(sql_alchemy_fixture, access_domain_fixture):
    """A particular SystemAccount can see its own AddressBooks as well as all the AddressBooks
       it is explicitly allowed to see, but no other AddressBooks."""


    account = access_domain_fixture.account
    address_book = access_domain_fixture.address_book
    other_address_book = access_domain_fixture.other_address_book

    unrelated_account = access_domain_fixture.new_account(email='unrelated@some.org')
    unrelated_address_book = access_domain_fixture.new_address_book(owner=unrelated_account)

    other_address_book.allow(account)

    # Checks to see whether an AddressBook is visible
    assert address_book.is_visible_to(account)
    assert other_address_book.is_visible_to(account)
    assert not unrelated_address_book.is_visible_to(account)

    # Getting a list of visible AddressBooks (for populating the screen)
    books = AddressBook.address_books_visible_to(account)
    assert set(books) == {address_book, other_address_book}


@with_fixtures(SqlAlchemyFixture, AccessDomainFixture)
def test_collaborator_rights(sql_alchemy_fixture, access_domain_fixture):
    """When allowing an account to see another's AddressBook, the rights it has to the AddressBook are specified."""


    account = access_domain_fixture.account
    address_book = access_domain_fixture.address_book
    other_address_book = access_domain_fixture.other_address_book

    # Case: defaults
    other_address_book.allow(account)
    assert not other_address_book.can_be_edited_by(account)
    assert not other_address_book.can_be_added_to_by(account)

    # Case: rights specified
    other_address_book.allow(account, can_edit_addresses=True, can_add_addresses=True)
    assert other_address_book.can_be_edited_by(account)
    assert other_address_book.can_be_added_to_by(account)


@with_fixtures(SqlAlchemyFixture, AccessDomainFixture)
def test_adding_collaborators(sql_alchemy_fixture, access_domain_fixture):
    """The owner of an AddressBook may add collaborators to it."""


    account = access_domain_fixture.account
    address_book = access_domain_fixture.address_book
    other_address_book = access_domain_fixture.other_address_book

    assert address_book.collaborators_can_be_added_by(account)
    assert not other_address_book.collaborators_can_be_added_by(account)


@with_fixtures(WebFixture, AccessDomainFixture, AccessUIFixture)
def test_logging_in(web_fixture, access_domain_fixture, access_ui_fixture):
    """A user first sees only a login screen on the home page; after logging in,
       all the address books visible to the user appear."""

    browser = access_ui_fixture.browser
    account = access_domain_fixture.account
    address_book = access_domain_fixture.address_book

    other_address_book = access_domain_fixture.other_address_book
    other_address_book.allow(account)

    browser.open('/')
    browser.type(XPath.input_labelled('Email'), 'johndoe@some.org')
    browser.type(XPath.input_labelled('Password'), access_domain_fixture.password)
    browser.click(XPath.button_labelled('Log in'))

    assert browser.is_element_present(XPath.link().with_text('Address book of johndoe@some.org'))
    assert browser.is_element_present(XPath.link().with_text('Address book of other@some.org'))


@with_fixtures(WebFixture, AccessDomainFixture, AccessUIFixture)
def test_edit_and_add_own(web_fixture, access_domain_fixture, access_ui_fixture):
    """The owner of an AddressBook can add and edit Addresses to the owned AddressBook."""


    browser = access_ui_fixture.browser
    fixture = access_domain_fixture
    account = fixture.account
    address_book = fixture.address_book

    web_fixture.log_in(browser=browser, system_account=account)
    browser.open('/')

    browser.click(XPath.link().with_text('Address book of johndoe@some.org'))

    # add
    browser.click(XPath.link().with_text('Add address'))
    browser.type(XPath.input_labelled('Name'), 'Someone')
    browser.type(XPath.input_labelled('Email'), 'someone@some.org')
    browser.click(XPath.button_labelled('Save'))

    assert browser.is_element_present(XPath.paragraph().including_text('Someone: someone@some.org'))

    # edit
    browser.click(XPath.button_labelled('Edit'))
    browser.type(XPath.input_labelled('Name'), 'Else')
    browser.type(XPath.input_labelled('Email'), 'else@some.org')
    browser.click(XPath.button_labelled('Update'))

    assert browser.is_element_present(XPath.paragraph().including_text('Else: else@some.org'))


# ------- Tests added for access control


@with_fixtures(WebFixture, AccessDomainFixture, AccessUIFixture)
def test_see_other(web_fixture, access_domain_fixture, access_ui_fixture):
    """If allowed, an account may see another account's AddressBook, and could edit or add Addresses,
       depending on the allowed rights."""

    browser = access_ui_fixture.browser
    account = access_domain_fixture.account
    address_book = access_domain_fixture.address_book

    other_address_book = access_domain_fixture.other_address_book
    other_address_book.allow(account)
    Address(address_book=other_address_book, email_address='friend@some.org', name='Friend').save()

    web_fixture.log_in(browser=browser, system_account=account)
    browser.open('/')

    browser.click(XPath.link().with_text('Address book of other@some.org'))

    assert browser.is_element_present(XPath.paragraph().including_text('Friend: friend@some.org'))

    # Case: can only see
    assert not browser.is_element_enabled(XPath.link().with_text('Add address'))
    assert not browser.is_element_enabled(XPath.button_labelled('Edit'))

    # Case: can edit only
    other_address_book.allow(account, can_edit_addresses=True, can_add_addresses=False)
    browser.refresh()
    assert not browser.is_element_enabled(XPath.link().with_text('Add address'))
    assert browser.is_element_enabled(XPath.button_labelled('Edit'))

    # Case: can add, and therefor also edit
    other_address_book.allow(account, can_add_addresses=True)
    browser.refresh()
    assert browser.is_element_enabled(XPath.link().with_text('Add address'))
    assert browser.is_element_enabled(XPath.button_labelled('Edit'))

    browser.click(XPath.button_labelled('Edit'))
    browser.type(XPath.input_labelled('Name'), 'Else')
    browser.type(XPath.input_labelled('Email'), 'else@some.org')
    browser.click(XPath.button_labelled('Update'))

    assert browser.is_element_present(XPath.paragraph().including_text('Else: else@some.org'))


@with_fixtures(WebFixture, AccessDomainFixture, AccessUIFixture)
def test_edit_other(web_fixture, access_domain_fixture, access_ui_fixture):
    """If you may only edit (not add) an address, then you may only edit the email address, not the name."""


    browser = access_ui_fixture.browser
    address_book = access_domain_fixture.address_book
    account = access_domain_fixture.account

    other_address_book = access_domain_fixture.other_address_book
    other_address_book.allow(account, can_edit_addresses=True, can_add_addresses=True)
    Address(address_book=other_address_book, email_address='friend@some.org', name='Friend').save()

    web_fixture.log_in(browser=browser, system_account=account)
    browser.open('/')

    browser.click(XPath.link().with_text('Address book of other@some.org'))
    browser.click(XPath.button_labelled('Edit'))

    # Case: may edit name
    assert browser.is_element_enabled(XPath.input_labelled('Name'))
    assert browser.is_element_enabled(XPath.input_labelled('Email'))

    # Case: may not edit name
    other_address_book.allow(account, can_edit_addresses=True, can_add_addresses=False  )
    browser.refresh()
    assert not browser.is_element_enabled(XPath.input_labelled('Name'))
    assert browser.is_element_enabled(XPath.input_labelled('Email'))

    browser.type(XPath.input_labelled('Email'), 'else@some.org')
    browser.click(XPath.button_labelled('Update'))

    assert browser.is_element_present(XPath.paragraph().including_text('Friend: else@some.org'))


@with_fixtures(WebFixture, AccessDomainFixture, AccessUIFixture)
def test_add_collaborator(web_fixture, access_domain_fixture, access_ui_fixture):
    """A user may add other users as collaborators to his address book, specifying the privileges in the process."""
    
    browser = access_ui_fixture.browser
    address_book = access_domain_fixture.address_book
    account = access_domain_fixture.account

    other_address_book = access_domain_fixture.other_address_book
    other_account = access_domain_fixture.other_account

    web_fixture.log_in(browser=browser, system_account=account)
    browser.open('/')

    assert address_book not in AddressBook.address_books_visible_to(other_account)
    assert not address_book.can_be_edited_by(other_account)
    assert not address_book.can_be_added_to_by(other_account)

    browser.click(XPath.link().with_text('Address book of johndoe@some.org'))
    browser.click(XPath.link().with_text('Add collaborator'))

    browser.select(XPath.select_labelled('Choose collaborator'), 'other@some.org')
    browser.click(XPath.input_labelled('May add new addresses'))
    browser.click(XPath.input_labelled('May edit existing addresses'))

    browser.click(XPath.button_labelled('Share'))

    assert access_ui_fixture.is_on_address_book_page_of('johndoe@some.org')

    assert address_book in AddressBook.address_books_visible_to(other_account)
    assert address_book.can_be_edited_by(other_account)
    assert address_book.can_be_added_to_by(other_account)


@uses(reahl_system_fixture=ReahlSystemFixture, access_domain_fixture=AccessDomainFixture)
class ViewScenarios(Fixture):
    @scenario
    def viewing_other_address_book(self):
        address_book = self.access_domain_fixture.other_address_book; Session.flush()
        self.url = '/address_book/%s' % address_book.id
        self.get = True

    @scenario
    def add_address_to_other_address_book(self):
        address_book = self.access_domain_fixture.other_address_book; Session.flush()
        self.url = '/add_address/%s' % address_book.id
        self.get = False

    @scenario
    def edit_address_in_other_address_book(self):
        address = Address(address_book=self.access_domain_fixture.other_address_book, email_address='somefriend@some.org', name='Friend')
        address.save()
        Session.flush()
        self.url = '/edit_address/%s' % address.id
        self.get = True

    @scenario
    def add_collaborator_to_other_address_book(self):
        address_book = self.access_domain_fixture.other_address_book; Session.flush()
        self.url = '/add_collaborator/%s' % address_book.id
        self.get = True


@with_fixtures(WebFixture, AccessUIFixture, AccessDomainFixture, ViewScenarios)
def test_view_permissions(web_fixture, access_ui_fixture, access_domain_fixture, view_scenarios):

    browser = access_ui_fixture.browser
    account = access_domain_fixture.account

    web_fixture.log_in(browser=browser, system_account=account)
    if view_scenarios.get:
        browser.open(view_scenarios.url, status=403)
    else:
        browser.post(view_scenarios.url, {}, status=403)




from reahl.doc.examples.howtos.customisingerrorpages.customisingerrorpages import BreakingUIWithBuiltInErrorPage, BreakingUIWithCustomErrorPage

web.site_root = BreakingUIWithBuiltInErrorPage
#web.site_root = BreakingUIWithCustomErrorPage




from reahl.tofu.pytestsupport import with_fixtures
from reahl.tofu import Fixture, uses

from reahl.web_dev.fixtures import WebFixture
from reahl.browsertools.browsertools import XPath

from reahl.doc.examples.howtos.optimisticconcurrency.optimisticconcurrency import OptimisticConcurrencyUI

@uses(web_fixture=WebFixture)
class OptimisticConcurrencyExampleFixture(Fixture):
    def new_browser(self):
        return self.web_fixture.driver_browser

    def check_counter_value_is(self, counter_value):
        assert self.browser.is_element_present(XPath.paragraph().with_text('Counter: %s' % counter_value))

    def link_opens_new_tab(self):
        return XPath.link().with_text('Open another tab...')

    def increment_button(self):
        return XPath.button_labelled('Increment')

    def submit_button(self):
        return XPath.button_labelled('Submit')

    def check_concurrency_error_is_displayed(self):
        alert = XPath.div().including_class('alert')
        assert 'Some data changed since you opened this page' in self.browser.get_text(alert)


@with_fixtures(WebFixture, OptimisticConcurrencyExampleFixture)
def test_example(web_fixture, optimistic_concurrency_example_fixture):
    fixture = optimistic_concurrency_example_fixture

    wsgi_application = web_fixture.new_wsgi_app(site_root=OptimisticConcurrencyUI, enable_js=True)
    web_fixture.reahl_server.set_app(wsgi_application)

    browser = fixture.browser

    browser.open('/')

    with browser.new_tab_closed():
        browser.click(fixture.link_opens_new_tab())
        browser.switch_to_different_tab()
        browser.click(fixture.increment_button())
        fixture.check_counter_value_is(1)

    # Back in the original tab, Cause the concurrency error
    fixture.check_counter_value_is(0) #The other tab just made this 1
    browser.click(fixture.submit_button())
    fixture.check_concurrency_error_is_displayed()








from reahl.web.fw import UserInterface
from reahl.web.layout import PageLayout
from reahl.web.bootstrap.page import HTML5Page
from reahl.web.bootstrap.ui import P, H, Div
from reahl.web.bootstrap.grid import ResponsiveSize, ColumnLayout, ColumnOptions, Container
from reahl.web.bootstrap.pagination import PagedPanel, SequentialPageIndex, PageMenu


class AddressBookUI(UserInterface):
    def assemble(self):
        page_layout = PageLayout(document_layout=Container(),
                                 contents_layout=ColumnLayout(ColumnOptions('main', size=ResponsiveSize(lg=6))).with_slots())
        self.define_page(HTML5Page).use_layout(page_layout)
        find = self.define_view('/', title='Addresses')
        find.set_slot('main', AddressBookPanel.factory())


class AddressBookPanel(Div):
    def __init__(self, view):
        super().__init__(view)

        self.add_child(H(view, 1, text='Addresses'))

        self.page_index = SequentialPageIndex(Address.all_addresses(), items_per_page=5)
        self.address_list = AddressList(view, self.page_index)
        self.page_menu = PageMenu(view, 'page_menu', self.page_index, self.address_list)

        self.add_children([self.page_menu, self.address_list])


class AddressList(PagedPanel):
    def __init__(self, view, page_index):
        super().__init__(view, page_index, 'addresslist')
        for address in self.current_contents:
            self.add_child(P(view, text='%s: %s' % (address.name, address.email_address)))


class Address:
    def __init__(self, name, email_address):
        self.name = name
        self.email_address = email_address

    @classmethod
    def all_addresses(cls):
        return [Address('friend %s' % i,'friend%s@some.org' % i ) for i in list(range(200))]



# Copyright 2013-2023 Reahl Software Services (Pty) Ltd. All rights reserved.
#
#    This file is part of Reahl.
#
#    Reahl is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as
#    published by the Free Software Foundation; version 3 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.



from sqlalchemy import Column, ForeignKey, UnicodeText, Integer, LargeBinary
from sqlalchemy.orm import relationship
from reahl.sqlalchemysupport import Session, Base
from reahl.component.modelinterface import ExposedNames, EmailField, Field, Event, Action, FileField
from reahl.web.fw import UserInterface
from reahl.web.layout import PageLayout
from reahl.web.bootstrap.page import HTML5Page
from reahl.web.bootstrap.ui import FieldSet, Div, P
from reahl.web.bootstrap.files import FileUploadInput
from reahl.web.bootstrap.grid import Container, ColumnLayout, ColumnOptions, ResponsiveSize
from reahl.web.bootstrap.forms import Form, FormLayout, ButtonInput, TextInput



class FileUploadUI(UserInterface):
    def assemble(self):
        self.define_page(HTML5Page).use_layout(PageLayout(document_layout=Container(),
                                                          contents_layout=ColumnLayout(
                                                              ColumnOptions('main', size=ResponsiveSize(lg=6))).with_slots()))
        home = self.define_view('/', title='File upload demo')
        home.set_slot('main', CommentPostPanel.factory())


class Comment(Base):
    __tablename__ = 'fileupload_comment'

    id = Column(Integer, primary_key=True)
    email_address  = Column(UnicodeText)
    text           = Column(UnicodeText)
    attached_files = relationship('AttachedFile', backref='comment')
    
    fields = ExposedNames()
    fields.email_address = lambda i: EmailField(label='Email address', required=True)
    fields.text          = lambda i: Field(label='Comment', required=True)
    fields.uploaded_files = lambda i: FileField(allow_multiple=True, max_size_bytes=4*1000*1000, max_files=4, accept=['text/*'])

    events = ExposedNames()
    events.submit = lambda i: Event(label='Submit', action=Action(i.submit))

    def attach_uploaded_files(self):
        for f in self.uploaded_files:
            self.attached_files.append(AttachedFile(comment=self, filename=f.filename, contents=f.contents))

    def submit(self):
        self.attach_uploaded_files()
        Session.add(self)


class AttachedFile(Base):
    __tablename__ = 'fileupload_attached_file'

    id = Column(Integer, primary_key=True)
    filename = Column(UnicodeText)
    contents = Column(LargeBinary)
    comment_id = Column(Integer, ForeignKey(Comment.id))


class CommentPostPanel(Div):
    def __init__(self, view):
        super().__init__(view)

        self.add_child(CommentForm(view))

        for comment in Session.query(Comment).all():
            self.add_child(CommentBox(view, comment))


class CommentForm(Form):
    def __init__(self, view):
        super().__init__(view, 'myform')

        new_comment = Comment()
        grouped_inputs = self.add_child(FieldSet(view, legend_text='Leave a comment'))
        grouped_inputs.use_layout(FormLayout())
        grouped_inputs.layout.add_input(TextInput(self, new_comment.fields.email_address))
        grouped_inputs.layout.add_input(TextInput(self, new_comment.fields.text))


        attachments = self.add_child(FieldSet(view, legend_text='Attach files'))
        attachments.use_layout(FormLayout())
        attachments.layout.add_input(FileUploadInput(self, new_comment.fields.uploaded_files), hide_label=True)

        self.define_event_handler(new_comment.events.submit)
        self.add_child(ButtonInput(self, new_comment.events.submit, style='primary'))


class CommentBox(Div):
    def __init__(self, view, comment):
        super().__init__(view)
        self.add_child(P(view, text='By %s: %s' % (comment.email_address, comment.text)))
        







from reahl.tofu import Fixture, uses
from reahl.tofu.pytestsupport import with_fixtures
from reahl.browsertools.browsertools import Browser, XPath
from reahl.doc.examples.tutorial.parameterised2.parameterised2 import AddressBookUI, Address
from reahl.web_dev.fixtures import WebFixture


@uses(web_fixture=WebFixture)
class AddressAppFixture(Fixture):

    def new_browser(self):
        return Browser(self.web_fixture.new_wsgi_app(site_root=AddressBookUI))

    def new_existing_address(self):
        address = Address(name='John Doe', email_address='johndoe@some.org')
        address.save()
        return address

    def is_on_home_page(self):
        return self.browser.title == 'Show'
        
    def is_on_add_page(self):
        return self.browser.title == 'Add'

    def is_on_edit_page_for(self, address):
        return self.browser.title == 'Edit %s' % address.name

    def address_is_listed_as(self, name, email_address):
        return self.browser.is_element_present(XPath.paragraph().including_text('%s: %s' % (name, email_address)))


@with_fixtures(WebFixture, AddressAppFixture)
def test_adding_an_address(web_fixture, address_app_fixture):
    """To add a new address, a user clicks on "Add Address" link on the menu, then supplies the 
       information for the new address and clicks the Save button. Upon success addition of the
       address, the user is returned to the home page where the new address is now listed."""

    browser = address_app_fixture.browser

    browser.open('/')
    browser.click(XPath.link().with_text('Add'))

    assert address_app_fixture.is_on_add_page()
    browser.type(XPath.input_labelled('Name'), 'John Doe')
    browser.type(XPath.input_labelled('Email'), 'johndoe@some.org')
    browser.click(XPath.button_labelled('Save'))

    assert address_app_fixture.is_on_home_page()
    assert address_app_fixture.address_is_listed_as('John Doe', 'johndoe@some.org') 


@with_fixtures(WebFixture, AddressAppFixture)
def test_editing_an_address(web_fixture, address_app_fixture):
    """To edit an existing address, a user clicks on the "Edit" button next to the chosen Address
       on the "Addresses" page. The user is then taken to an "Edit" View for the chosen Address and 
       can change the name or email address. Upon clicking the "Update" Button, the user is sent back 
       to the "Addresses" page where the changes are visible."""

    browser = address_app_fixture.browser
    existing_address = address_app_fixture.existing_address

    browser.open('/')
    browser.click(XPath.button_labelled('Edit'))

    assert address_app_fixture.is_on_edit_page_for(existing_address)
    browser.type(XPath.input_labelled('Name'), 'John Doe-changed')
    browser.type(XPath.input_labelled('Email'), 'johndoe@some.changed.org')
    browser.click(XPath.button_labelled('Update'))

    assert address_app_fixture.is_on_home_page()
    assert address_app_fixture.address_is_listed_as('John Doe-changed', 'johndoe@some.changed.org') 
                








from reahl.web.fw import UserInterface
from reahl.web.layout import PageLayout
from reahl.web.bootstrap.page import HTML5Page
from reahl.web.bootstrap.ui import P
from reahl.web.bootstrap.forms import Form, TextInput, Button, FormLayout, PasswordInput
from reahl.web.bootstrap.navs import Nav, TabLayout
from reahl.web.bootstrap.grid import ColumnLayout, ColumnOptions, ResponsiveSize, Container
from reahl.domain.systemaccountmodel import AccountManagementInterface, LoginSession



class MenuPage(HTML5Page):
    def __init__(self, view, main_bookmarks):
        super().__init__(view)
        self.use_layout(PageLayout(document_layout=Container()))
        contents_layout = ColumnLayout(ColumnOptions('main', size=ResponsiveSize(md=4))).with_slots()
        self.layout.contents.use_layout(contents_layout)
        self.layout.header.add_child(Nav(view).use_layout(TabLayout()).with_bookmarks(main_bookmarks))


class LoginForm(Form):
    def __init__(self, view):
        super().__init__(view, 'login')
        self.use_layout(FormLayout())
        accounts = AccountManagementInterface.for_current_session()

        if self.exception:
            self.layout.add_alert_for_domain_exception(self.exception)

        self.layout.add_input(TextInput(self, accounts.fields.email))
        self.layout.add_input(PasswordInput(self, accounts.fields.password))

        self.define_event_handler(accounts.events.login_event)
        self.add_child(Button(self, accounts.events.login_event, style='primary'))


class LoginUI(UserInterface):
    def assemble(self):
        login_session = LoginSession.for_current_session()
        if login_session.account:
            logged_in_as = login_session.account.email
        else:
            logged_in_as = 'Guest'

        home = self.define_view('/', title='Home')
        home.set_slot('main', P.factory(text='Welcome %s' % logged_in_as))

        login_page = self.define_view('/login', title='Log in')
        login_page.set_slot('main', LoginForm.factory())
        
        bookmarks = [i.as_bookmark(self) for i in [home, login_page]]
        self.define_page(MenuPage, bookmarks)









# Copyright 2019-2023 Reahl Software Services (Pty) Ltd. All rights reserved.
#
#    This file is part of Reahl.
#
#    Reahl is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as
#    published by the Free Software Foundation; version 3 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.



from reahl.component.modelinterface import ExposedNames, Field, Event, Action, Choice, ChoiceField, IntegerField
from reahl.component.exceptions import DomainException
from reahl.web.fw import UserInterface
from reahl.web.ui import StaticColumn, DynamicColumn
from reahl.web.layout import PageLayout
from reahl.web.bootstrap.page import HTML5Page
from reahl.web.bootstrap.ui import FieldSet, Div, P
from reahl.web.bootstrap.grid import Container, ColumnLayout, ColumnOptions, ResponsiveSize
from reahl.web.bootstrap.forms import Form, FormLayout, TextInput, RadioButtonSelectInput, Button
from reahl.web.bootstrap.tables import Table

from sqlalchemy import Column, Integer, UnicodeText, ForeignKey
from sqlalchemy.orm import relationship
from reahl.sqlalchemysupport import Base, Session, session_scoped


class DynamicUI(UserInterface):
    def assemble(self):
        self.define_page(HTML5Page).use_layout(PageLayout(document_layout=Container(),
                                                          contents_layout=ColumnLayout(
                                                              ColumnOptions('main', size=ResponsiveSize(lg=6))).with_slots()))
        home = self.define_view('/', title='Dynamic content demo')
        home.set_slot('main', AllocationDetailForm.factory())


class AllocationDetailForm(Form):
    def __init__(self, view):
        super().__init__(view, 'investment_order_allocation_details_form')
        self.use_layout(FormLayout())

        self.investment_order = InvestmentOrder.for_current_session()
        self.enable_refresh(on_refresh=self.investment_order.events.allocation_changed)

        self.add_allocation_controls()
        self.add_allocation_table()

        self.define_event_handler(self.investment_order.events.submit)
        self.add_child(Button(self, self.investment_order.events.submit))

    def add_allocation_controls(self):
        allocation_controls = self.add_child(FieldSet(self.view, legend_text='Investment allocation'))
        allocation_controls.use_layout(FormLayout())

        if self.exception:
            self.layout.add_alert_for_domain_exception(self.exception)
        
        total_amount_input = TextInput(self, self.investment_order.fields.amount, refresh_widget=self)
        allocation_controls.layout.add_input(total_amount_input)

        amount_or_percentage_radio = RadioButtonSelectInput(self, self.investment_order.fields.amount_or_percentage, refresh_widget=self)
        allocation_controls.layout.add_input(amount_or_percentage_radio)

    def make_allocation_input(self, allocation, field):
        div = Div(self.view).use_layout(FormLayout())
        div.layout.add_input(TextInput(self, field.with_discriminator(allocation.fund_code), refresh_widget=self), hide_label=True)
        return div

    def make_total_widget(self, total_value):
        return P(self.view, text=str(total_value))

    def add_allocation_table(self):
        def make_amount_input(view, allocation):
            return self.make_allocation_input(allocation, allocation.fields.amount)

        def make_percentage_input(view, allocation):
            return self.make_allocation_input(allocation, allocation.fields.percentage)

        def make_percentage_total(view, investment_order):
            return self.make_total_widget(investment_order.total_allocation_percentage)

        def make_amount_total(view, investment_order):
            return self.make_total_widget(investment_order.total_allocation_amount)

        columns = [StaticColumn(Field(label='Fund'), 'fund', footer_label='Totals')]
        columns.append(DynamicColumn('Percentage', make_percentage_input, make_footer_widget=make_percentage_total))
        columns.append(DynamicColumn('Amount', make_amount_input, make_footer_widget=make_amount_total))
        table = Table(self.view).with_data(columns, self.investment_order.allocations,
                                           footer_items=[self.investment_order])
        self.add_child(table)


@session_scoped
class InvestmentOrder(Base):
    __tablename__ = 'dynamic_content_investment_order'

    id              = Column(Integer, primary_key=True)
    amount          = Column(Integer)
    amount_or_percentage = Column(UnicodeText)
    allocations     = relationship('reahl.doc.examples.tutorial.dynamiccontent.dynamiccontent.Allocation',
                                   back_populates='investment_order', lazy='immediate', cascade="all, delete-orphan")

    fields = ExposedNames()
    fields.amount                = lambda i: IntegerField(label='Total amount', required=True)
    fields.amount_or_percentage  = lambda i: ChoiceField([Choice('amount', Field(label='Amount')),
                                                          Choice('percentage', Field(label='Percentage'))],
                                                         label='Allocate using', required=True)

    events = ExposedNames()
    events.submit = lambda i: Event(label='Submit', action=Action(i.submit))
    events.allocation_changed = lambda i: Event(action=Action(i.recalculate))

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.clear()

    def clear(self):
        self.amount_or_percentage = 'percentage'
        self.amount = 0
        self.allocations = [
            Allocation(self, 'Fund A'),
            Allocation(self, 'Fund B')]

    @property
    def is_in_percentage(self):
        return self.amount_or_percentage == 'percentage'

    def recalculate(self):
        for allocation in self.allocations:
            allocation.recalculate(self.amount)

    @property
    def total_allocation_amount(self):
       return sum([i.amount for i in self.allocations])

    @property
    def total_allocation_percentage(self):
        return sum([i.percentage for i in self.allocations])

    def validate_allocations(self):
        if self.is_in_percentage:
            if self.total_allocation_percentage != 100:
                raise DomainException(message='Please ensure allocation percentages add up to 100')
        else:
            if self.total_allocation_amount != self.amount:
                raise DomainException(message='Please ensure allocation amounts add up to your total amount (%s)' % self.amount)

    def submit(self):
        print('Submitting investment')
        self.recalculate()
        self.validate_allocations()

        print('\tAmount: %s' % self.amount)
        print('\tAllocations (%s)' % self.amount_or_percentage)
        for allocation in self.allocations:
            allocation_size = allocation.percentage if self.is_in_percentage else allocation.amount
            print('\t\tFund %s(%s): %s (%s)' % (allocation.fund, allocation.fund_code, allocation_size, allocation.amount))

        self.clear()


class Allocation(Base):
    __tablename__ = 'dynamic_content_allocation'

    id         = Column(Integer, primary_key=True)
    percentage = Column(Integer)
    amount     = Column(Integer)
    fund       = Column(UnicodeText)
    investment_order_id = Column(Integer, ForeignKey(InvestmentOrder.id))
    investment_order  = relationship('reahl.doc.examples.tutorial.dynamiccontent.dynamiccontent.InvestmentOrder', back_populates='allocations')

    fields = ExposedNames()
    fields.percentage    = lambda i: IntegerField(label='Percentage', required=True, writable=lambda field: i.is_in_percentage)
    fields.amount        = lambda i: IntegerField(label='Amount', required=True, writable=lambda field: i.is_in_amount)

    def __init__(self, investment_order, fund_name):
        super().__init__(investment_order=investment_order)
        self.fund = fund_name
        self.amount = 0
        self.percentage = 0

    @property
    def fund_code(self):
        return self.fund.replace(' ', '').upper()

    @property
    def is_in_percentage(self):
        return self.investment_order.is_in_percentage

    @property
    def is_in_amount(self):
        return not self.is_in_percentage

    def recalculate(self, total):
        if self.is_in_amount:
            if total == 0:
                self.percentage = 0
            else:
                self.percentage = int(round(self.amount / total * 100))
        else:
            self.amount = int(round(self.percentage / 100 * total))







from sqlalchemy import Column, Integer, UnicodeText, Boolean

from reahl.sqlalchemysupport import Session, Base

from reahl.web.fw import UserInterface, Widget
from reahl.web.layout import PageLayout
from reahl.web.bootstrap.page import HTML5Page
from reahl.web.bootstrap.ui import Div, P, H
from reahl.web.bootstrap.forms import Form, TextInput, Button, FormLayout, FieldSet
from reahl.web.bootstrap.grid import ColumnLayout, ColumnOptions, ResponsiveSize, Container
from reahl.component.modelinterface import ExposedNames, EmailField, Field, Event, Action


class AddressBookUI(UserInterface):
    def assemble(self):
        find = self.define_view('/', title='Addresses', page=AddressBookPage.factory())
        find.set_slot('main', AddressBookPanel.factory())


class AddressBookPage(HTML5Page):
    def __init__(self, view):
        super().__init__(view)
        self.use_layout(PageLayout(document_layout=Container()))
        contents_layout = ColumnLayout(ColumnOptions('main', size=ResponsiveSize(md=4))).with_slots()
        self.layout.contents.use_layout(contents_layout)


class AddressBookPanel(Div):
    def __init__(self, view):
        super().__init__(view)

        self.add_child(H(view, 1, text='Addresses'))

        for address in Session.query(Address).all():
            self.add_child(AddressBox(view, address))

        self.add_child(AddAddressForm(view))


class AddAddressForm(Form):
    def __init__(self, view):
        super().__init__(view, 'add_form')

        new_address = Address()

        grouped_inputs = self.add_child(FieldSet(view, legend_text='Add an address'))
        grouped_inputs.use_layout(FormLayout())
        grouped_inputs.layout.add_input(TextInput(self, new_address.fields.name))
        grouped_inputs.layout.add_input(TextInput(self, new_address.fields.email_address))

        self.define_event_handler(new_address.events.save)
        grouped_inputs.add_child(Button(self, new_address.events.save, style='primary'))


class AddressBox(Widget):
    def __init__(self, view, address):
        super().__init__(view)
        new = ' (new)' if address.added_today else ''
        self.add_child(P(view, text='%s: %s%s' % (address.name, address.email_address, new)))


class Address(Base):
    __tablename__ = 'jobsbootstrap_address'

    id            = Column(Integer, primary_key=True)
    email_address = Column(UnicodeText)
    name          = Column(UnicodeText)
    added_today   = Column(Boolean, nullable=False, default=True)

    @classmethod
    def clear_added_flags(cls):
        for address in Session.query(cls).filter_by(added_today=True):
            address.added_today = False

    fields = ExposedNames()
    fields.name = lambda i: Field(label='Name', required=True)
    fields.email_address = lambda i: EmailField(label='Email', required=True)

    events = ExposedNames()
    events.save = lambda i: Event(label='Save', action=Action(i.save))

    def save(self):
        Session.add(self)







# To run this test do:
# pytest 
#


from reahl.tofu import Fixture, uses
from reahl.tofu.pytestsupport import with_fixtures

from reahl.browsertools.browsertools import XPath
from reahl.doc.examples.howtos.ajaxbootstrap.ajaxbootstrap import WidgetRefreshUI

from reahl.web_dev.fixtures import WebFixture


@uses(web_fixture=WebFixture)
class RefreshFixture(Fixture):

    def new_browser(self):
        return self.web_fixture.driver_browser
        
    def new_wsgi_app(self):
        return self.web_fixture.new_wsgi_app(site_root=WidgetRefreshUI, enable_js=True)

    def text_shows_selected(self, expected_selected):
        return self.browser.is_element_present(XPath.paragraph().including_text('You selected link number %s' % expected_selected))


#------ Tests

@with_fixtures(WebFixture, RefreshFixture)
def test_refreshing_widget(web_fixture, refresh_fixture):
    """Clicking on a link, refreshes the displayed text to indicate which link 
       was clicked, without triggering a page load."""


    web_fixture.reahl_server.set_app(refresh_fixture.wsgi_app)
    browser = refresh_fixture.browser

    browser.open('/')

    assert refresh_fixture.text_shows_selected(1)
    assert not refresh_fixture.text_shows_selected(3)

    with browser.no_page_load_expected():
        browser.click(XPath.link().with_text('Select 3'))

    assert not refresh_fixture.text_shows_selected(1)
    assert refresh_fixture.text_shows_selected(3)




from reahl.web.fw import UserInterface, Widget
from reahl.web.bootstrap.page import HTML5Page
from reahl.web.bootstrap.ui import TextNode, Div, H, P
from reahl.web.bootstrap.navbar import Navbar, ResponsiveLayout
from reahl.web.bootstrap.grid import Container
from reahl.sqlalchemysupport import Session, Base
from sqlalchemy import Column, Integer, UnicodeText


class AddressBookPage(HTML5Page):
    def __init__(self, view):
        super().__init__(view)
        self.body.use_layout(Container())

        layout = ResponsiveLayout('md', colour_theme='dark', bg_scheme='primary')
        navbar = Navbar(view, css_id='my_nav').use_layout(layout)
        navbar.layout.set_brand_text('Address book')
        navbar.layout.add(TextNode(view, 'All your addresses in one place'))

        self.body.add_child(navbar)
        self.body.add_child(AddressBookPanel(view))


class AddressBookPanel(Div):
    def __init__(self, view):
        super().__init__(view)

        self.add_child(H(view, 1, text='Addresses'))

        for address in Session.query(Address).all():
            self.add_child(AddressBox(view, address))


class AddressBox(Widget):
    def __init__(self, view, address):
        super().__init__(view)
        self.add_child(P(view, text='%s: %s' % (address.name, address.email_address)))


class AddressBookUI(UserInterface):
    def assemble(self):
        self.define_view('/', title='Address book', page=AddressBookPage.factory())


class Address(Base):
    __tablename__ = 'addressbook1_address'

    id            = Column(Integer, primary_key=True)
    email_address = Column(UnicodeText)
    name          = Column(UnicodeText)

    def save(self):
        Session.add(self)






from reahl.web.fw import UserInterface
from reahl.web.bootstrap.page import HTML5Page
from reahl.web.bootstrap.ui import TextNode
from reahl.web.bootstrap.navbar import Navbar, ResponsiveLayout
from reahl.web.bootstrap.grid import Container


class AddressBookPage(HTML5Page):
    def __init__(self, view):
        super().__init__(view)
        self.body.use_layout(Container())

        layout = ResponsiveLayout('md', colour_theme='dark', bg_scheme='primary')
        navbar = Navbar(view, css_id='my_nav').use_layout(layout)
        navbar.layout.set_brand_text('Address book')
        navbar.layout.add(TextNode(view, 'All your addresses in one place'))

        self.body.add_child(navbar)


class AddressBookUI(UserInterface):
    def assemble(self):
        self.define_view('/', title='Address book', page=AddressBookPage.factory())




# To run this test do:
# pytest
#
# To set up a demo database for playing with, do:
# pytest -o python_functions=demo_setup


from reahl.tofu.pytestsupport import with_fixtures

from reahl.doc.examples.tutorial.addressbook2bootstrap.addressbook2bootstrap import Address

from reahl.sqlalchemysupport_dev.fixtures import SqlAlchemyFixture


@with_fixtures(SqlAlchemyFixture)
def demo_setup(sql_alchemy_fixture):
    sql_alchemy_fixture.commit = True
    Address(email_address='friend1@some.org', name='Friend1').save()
    Address(email_address='friend2@some.org', name='Friend2').save()
    Address(email_address='friend3@some.org', name='Friend3').save()
    Address(email_address='friend4@some.org', name='Friend4').save()





from reahl.web.fw import UserInterface
from reahl.web.bootstrap.page import HTML5Page
from reahl.web.bootstrap.forms import Form, TextInput, FormLayout, ButtonInput
from reahl.component.modelinterface import ExposedNames, EmailField, Event

class ValidationUI(UserInterface):
    def assemble(self):
        self.define_view('/', title='Validation demo', page=HomePage.factory())


class HomePage(HTML5Page):
    def __init__(self, view):
        super().__init__(view)
        self.body.add_child(CommentForm(view))


class Comment:
    fields = ExposedNames()
    fields.email_address = lambda i: EmailField(label='Email address', required=True)
    
    events = ExposedNames()
    events.do_nothing = lambda i: Event(label='Submit')


class CommentForm(Form):
    def __init__(self, view):
        super().__init__(view, 'myform')

        new_comment = Comment()
        self.use_layout(FormLayout())
        self.layout.add_input( TextInput(self, new_comment.fields.email_address) )

        self.define_event_handler(new_comment.events.do_nothing)
        self.add_child(ButtonInput(self, new_comment.events.do_nothing))








from reahl.web.fw import UserInterface
from reahl.web.holder.holder import PlaceholderImage
from reahl.web.bootstrap.page import HTML5Page
from reahl.web.bootstrap.ui import P
from reahl.web.bootstrap.grid import Container
from reahl.web.bootstrap.carousel import Carousel


class MyPage(HTML5Page):
    def __init__(self, view):
        super().__init__(view)
        self.body.use_layout(Container())

        carousel = Carousel(view, 'my_example_carousel_id', show_indicators=True)
        self.body.add_child(carousel)

        carousel.add_slide(PlaceholderImage(view, 900, 500, text='Slide 1', alt='Slide 1 was here'),
                           caption_widget=P(view, text='a paragraph with text'))

        carousel.add_slide(PlaceholderImage(view, 900, 500, text='Slide 2', alt='Slide 2 was here'),
                           caption_widget=P(view, text='a different paragraph'))


class CarouselUI(UserInterface):
    def assemble(self):
        self.define_view('/', title='Carousel demo', page=MyPage.factory())








from reahl.tofu import Fixture, uses
from reahl.tofu.pytestsupport import with_fixtures

from reahl.browsertools.browsertools import Browser, XPath

from reahl.doc.examples.tutorial.tablebootstrap.tablebootstrap import AddressBookUI, Address

from reahl.sqlalchemysupport_dev.fixtures import SqlAlchemyFixture
from reahl.web_dev.fixtures import WebFixture


@uses(web_fixture=WebFixture)
class TableExampleFixture(Fixture):

    def new_browser(self):
        return Browser(self.web_fixture.new_wsgi_app(site_root=AddressBookUI))

    def create_addresses(self):
        addresses = [Address(name='friend %s' % i, email_address='friend%s@some.org' % i) for i in list(range(1, 21))]
        for address in addresses:
            address.save()
        return addresses

    def is_on_edit_page_for(self, address_name):
        return self.browser.title == 'Edit %s' % address_name

    def is_on_home_page(self):
        return self.browser.title == 'Addresses'

    def address_is_listed_as(self, name):
        return self.browser.is_element_present(XPath.table_cell().with_text(name))


@with_fixtures(SqlAlchemyFixture, TableExampleFixture)
def demo_setup(sql_alchemy_fixture, fixture):
    sql_alchemy_fixture.commit = True

    fixture.create_addresses()


@with_fixtures(WebFixture, TableExampleFixture)
def test_editing_an_address(web_fixture, fixture):
    """To edit an existing address, a user clicks on the "Edit" link next to the chosen Address
       on the "Addresses" page. The user is then taken to an "Edit" View for the chosen Address and
       can change the name or email address. Upon clicking the "Update" Button, the user is sent back
       to the "Addresses" page where the changes are visible."""

    all_addresses = fixture.create_addresses()    #create some data to play with

    original_address_name = 'friend 7'   #choose the seventh address to edit

    fixture.browser.open('/')
    fixture.browser.click(XPath.link().with_text('Edit').inside_of(XPath.table_row()[7]))

    assert fixture.is_on_edit_page_for(original_address_name)
    fixture.browser.type(XPath.input_labelled('Name'), 'John Doe-changed')
    fixture.browser.type(XPath.input_labelled('Email'), 'johndoe@some.changed.org')
    fixture.browser.click(XPath.button_labelled('Update'))

    assert fixture.is_on_home_page()
    assert fixture.address_is_listed_as('John Doe-changed')
    assert fixture.address_is_listed_as('johndoe@some.changed.org')
    assert not fixture.address_is_listed_as(original_address_name)


@with_fixtures(WebFixture, TableExampleFixture)
def test_deleting_several_address(web_fixture, fixture):
    """To delete several address, a user "checks" the box next to each of the Addresses
       on the "Addresses" page she wants to delete. Upon clicking the "Delete Selected" Button, the page
       refreshes, and the remaining addresses appear."""

    fixture.create_addresses()    #create some data to play with


    web_fixture.reahl_server.set_app(web_fixture.new_wsgi_app(site_root=AddressBookUI))

    fixture.browser = web_fixture.driver_browser
    fixture.browser.open('/')

    name_of_address_1 = 'friend 1'
    name_of_address_13 = 'friend 13'
    name_of_address_20 = 'friend 20'

    assert fixture.address_is_listed_as(name_of_address_1)
    assert fixture.address_is_listed_as(name_of_address_13)
    assert fixture.address_is_listed_as(name_of_address_20)
   
    fixture.browser.click(XPath.checkbox().inside_of(XPath.table_row()[1].inside_of(XPath.table_body())))
    fixture.browser.click(XPath.checkbox().inside_of(XPath.table_row()[13].inside_of(XPath.table_body())))
    fixture.browser.click(XPath.checkbox().inside_of(XPath.table_row()[20].inside_of(XPath.table_body())))
    fixture.browser.click(XPath.button_labelled('Delete Selected'))

    assert fixture.is_on_home_page()
    assert not fixture.address_is_listed_as(name_of_address_1)
    assert not fixture.address_is_listed_as(name_of_address_13)
    assert not fixture.address_is_listed_as(name_of_address_20)






from reahl.tofu import Fixture, uses
from reahl.tofu.pytestsupport import with_fixtures

from reahl.browsertools.browsertools import Browser, XPath

from reahl.doc.examples.tutorial.componentconfigbootstrap.componentconfigbootstrap import AddressBookUI, Address

from reahl.web_dev.fixtures import WebFixture


@uses(web_fixture=WebFixture)
class ConfigFixture(Fixture):

    def new_wsgi_app(self):
        return self.web_fixture.new_wsgi_app(site_root=AddressBookUI)

    def new_existing_address(self):
        address = Address(name='John Doe', email_address='johndoe@some.org')
        address.save()
        return address

    def new_browser(self):
        return Browser(self.wsgi_app)

    def address_is_listed_as(self, name, email_address):
        return self.browser.is_element_present(XPath.paragraph().including_text('%s: %s' % (name, email_address)))

    def heading_is_displayed(self):
        return self.browser.is_element_present(XPath.heading(1).with_text('Addresses'))


@with_fixtures(WebFixture, ConfigFixture)
def test_add_address(web_fixture, config_fixture):
    """A user can add an address, after which the address is listed."""

    browser = config_fixture.browser

    browser.open('/')
    browser.type(XPath.input_labelled('Name'), 'John')
    browser.type(XPath.input_labelled('Email'), 'johndoe@some.org')

    browser.click(XPath.button_labelled('Save'))

    assert config_fixture.address_is_listed_as('John', 'johndoe@some.org')


@with_fixtures(WebFixture)
def test_config_was_read_from_file(web_fixture):
    """The tests are run with config read from an actual config file, not the default config."""
    assert web_fixture.context.config.componentconfig.showheader == True


@with_fixtures(WebFixture, ConfigFixture)
def test_configurable_heading(web_fixture, config_fixture):
    """Whether the heading is displayed or not, is configurable."""

    browser = config_fixture.browser

    web_fixture.context.config.componentconfig.showheader = False
    browser.open('/')
    assert not config_fixture.heading_is_displayed()

    web_fixture.context.config.componentconfig.showheader = True
    browser.open('/')
    assert config_fixture.heading_is_displayed()




# In production this has to be set, to the name of the egg of your application:
reahlsystem.root_egg = 'hellonginx'   

# If using SQLite:
reahlsystem.connection_uri = 'sqlite:////var/local/hellonginx/hellonginx.db'  

# If using PostgreSQL:
#reahlsystem.connection_uri = 'postgresql://hellonginx:hellonginx@localhost/hellonginx'  

reahlsystem.debug = False

#reahl.component.config.ConfigurationException: reahlsystem.debug has been defaulted to a value not suitable for production use: "True". You can set it in /etc/reahl.d/hellonginx/reahl.config.py



import sys
from reahl.component.config import StoredConfiguration
from reahl.component.dbutils import SystemControl
from reahl.sqlalchemysupport import Session, metadata
from reahl.component.context import ExecutionContext

from reahl.doc.examples.tutorial.migrationexamplebootstrap.migrationexamplebootstrap import Address

config = StoredConfiguration(sys.argv[1])
config.configure()
with ExecutionContext() as context:
    context.config = config
    context.system_control = SystemControl(config)

    try:
        context.system_control.orm_control.connect()
        metadata.create_all()

        Session.add(Address(name='John Doe', email_address='johndoe@some.org'))
        Session.add(Address(name='Jane Johnson', email_address='janejohnson@some.org'))
        Session.add(Address(name='Jack Black', email_address='jackblack@some.org'))

        context.system_control.orm_control.commit()

    finally:
        context.system_control.orm_control.disconnect()




import threading

from flaky import flaky

from reahl.tofu import Fixture
from reahl.tofu.pytestsupport import with_fixtures

from reahl.sqlalchemysupport import Session
from reahl.browsertools.browsertools import Browser, XPath

from reahl.doc.examples.tutorial.sessionscopebootstrap.sessionscopebootstrap import SessionScopeUI, User

from reahl.sqlalchemysupport_dev.fixtures import SqlAlchemyFixture
from reahl.web_dev.fixtures import WebFixture


class SessionScopeFixture(Fixture):
    password = 'topsecret'

    def new_user(self):
        user = User(name='John Doe', email_address='johndoe@some.org')
        user.set_password(self.password)
        Session.add(user)
        return user


@with_fixtures(SqlAlchemyFixture, SessionScopeFixture)
def demo_setup(sql_alchemy_fixture, session_scope_fixture):
    sql_alchemy_fixture.commit = True

    session_scope_fixture.new_user()


@with_fixtures(WebFixture, SessionScopeFixture)
def test_logging_in(web_fixture, session_scope_fixture):
    """A user can log in by going to the Log in page.
       The name of the currently logged in user is displayed on the home page."""

    browser = Browser(web_fixture.new_wsgi_app(site_root=SessionScopeUI))
    user = session_scope_fixture.user

    browser.open('/')
    browser.click(XPath.link().with_text('Log in'))

    browser.type(XPath.input_labelled('Email'), 'johndoe@some.org')
    browser.type(XPath.input_labelled('Password'), 'topsecret')
    browser.click(XPath.button_labelled('Log in'))

    browser.click(XPath.link().with_text('Home'))
    assert browser.is_element_present(XPath.paragraph().including_text('Welcome John Doe'))
    

@with_fixtures(WebFixture, SessionScopeFixture)
@flaky(max_runs=3, min_passes=1)
def test_email_retained(web_fixture, session_scope_fixture):
    """The email address used when last logged in is always pre-populated on the Log in page."""

    browser = Browser(web_fixture.new_wsgi_app(site_root=SessionScopeUI))
    user = session_scope_fixture.user

    browser.open('/')
    browser.click(XPath.link().with_text('Log in'))

    browser.type(XPath.input_labelled('Email'), 'johndoe@some.org')
    browser.type(XPath.input_labelled('Password'), 'topsecret')
    browser.click(XPath.button_labelled('Log in'))
    typed_value = browser.get_value(XPath.input_labelled('Email'))
    assert typed_value == 'johndoe@some.org'

    # Go away from the page, then back
    browser.click(XPath.link().with_text('Home'))
    browser.click(XPath.link().with_text('Log in'))

    # .. then the email is still pre-populated
    typed_value = browser.get_value(XPath.input_labelled('Email'))
    assert typed_value == 'johndoe@some.org'
    

@with_fixtures(WebFixture, SessionScopeFixture)
def test_domain_exception(web_fixture, session_scope_fixture):
    """Typing the wrong password results in an error message being shown to the user."""

    browser = Browser(web_fixture.new_wsgi_app(site_root=SessionScopeUI))
    user = session_scope_fixture.user

    browser.open('/')
    browser.click(XPath.link().with_text('Log in'))

    browser.type(XPath.input_labelled('Email'), 'johndoe@some.org')
    browser.type(XPath.input_labelled('Password'), 'wrong password')
    browser.click(XPath.button_labelled('Log in'))

    assert browser.is_element_present(XPath.div().including_text('The email/password given do not match'))
